@@ -43,6 +43,13 @@ class EMIOptionsEnum(int, Enum):
     NINE_MONTHS = 9
 
 
+class EMIOptionsResponseEnum(str, Enum):
+    NONE = 0
+    THREE_MONTHS = 3
+    SIX_MONTHS = 6
+    NINE_MONTHS = 9
+
+
 class ShippingMethodEnum(str, Enum):
     YES = "YES"
     NO = "NO"
@@ -439,9 +446,13 @@ class BaseOrderResponse(BaseModel):
     card_issuer_country_code: str
     currency_type: str
     currency_amount: Decimal
+    currency_rate: Decimal
     risk_level: RiskLevelEnum
     risk_title: str
 
+    error: Optional[str]
+    base_fair: Optional[Decimal]
+    card_sub_brand: Optional[str]
     value_a: Optional[str]
     value_b: Optional[str]
     value_c: Optional[str]
@@ -451,25 +462,36 @@ class BaseOrderResponse(BaseModel):
 class IPNResponse(BaseOrderResponse):
     """IPN response dataclass with validation"""
 
+    store_id: str
     status: IPNOrderStatusEnum
     verify_sign: str
     verify_key: str
+    verify_sign_sha2: Optional[str]
 
-    def validate_against_credential(self, credential: Union[Credential, dict]):
-        if not isinstance(credential, Credential):
-            credential = Credential(**credential)
+    def get_hash(self, credential: Credential, hasher=md5):
         keys = self.verify_key.split(",")
         keys.append("store_passwd")
         keys = sorted(keys)
         data = []
         for key in keys:
             if key == "store_passwd":
-                data.append((key, md5(credential.store_passwd).hexdigest))
+                data.append((key, hasher(credential.store_passwd.encode()).hexdigest()))
             else:
-                data.append((key, getattr(self, key)))
-        hash_string = "&".join([v.join("=") for v in data])
-        hash_string = md5(hash_string.encode()).hexdigest()
-        return hash_string == self.verify_sign
+                val = getattr(self, key)
+                if isinstance(val, Enum):
+                    val = val.value
+                data.append((key, str(val)))
+        hash_string = "&".join(["=".join(v) for v in data])
+        hash_string = hasher(hash_string.encode()).hexdigest()
+
+        return hash_string
+
+    def validate_against_credential(self, credential: Union[Credential, dict]):
+        if not isinstance(credential, Credential):
+            credential = Credential(**credential)
+        hash = self.get_hash(credential)
+
+        return hash == self.verify_sign
 
 
 class IPNValidationStatus(BaseModel):
@@ -504,8 +526,7 @@ class OrderValidationResponse(BaseOrderResponse):
     """Order validation response."""
 
     status: OrderStatusEnum
-    emi_instalment: EMIOptionsEnum
-    emi_amount: Decimal
+    emi_instalment: EMIOptionsResponseEnum
     discount_amount: Decimal
     discount_percentage: Decimal
     discount_remarks: str
@@ -601,6 +622,7 @@ class Session(BaseModel):
     risk_level: RiskLevelEnum
     risk_title: str
 
+    error: Optional[str]
     currency: Optional[str]
     emi_instalment: Optional[int]
     emi_amount: Optional[int]
